<?php

declare(strict_types=1);

namespace mirzaev\ebala\controllers;

// Файлы проекта
use mirzaev\ebala\controllers\core,
	mirzaev\ebala\controllers\traits\errors,
	mirzaev\ebala\models\task as model,
	mirzaev\ebala\models\account,
	mirzaev\ebala\models\worker,
	mirzaev\ebala\models\market,
	mirzaev\ebala\models\core as _core;

// Библиотека для ArangoDB
use ArangoDBClient\Document as _document;

// System libraries
use datetime,
	exception;

/**
 * Контроллер задания
 *
 * @package mirzaev\ebala\controllers
 * @author Arsen Mirzaev Tatyano-Muradovich <arsen@mirzaev.sexy>
 */
final class task extends core
{
	use errors;

	/**
	 * Создать
	 *
	 * @param array $parameters Параметры запроса
	 *
	 * @return void В буфер вывода JSON-документ с запрашиваемыми параметрами
	 */
	public function create(array $parameters = []): void
	{
		if ($this->account->status() && ($this->account->type === 'administrator' || $this->account->type === 'operator' || $this->account->type === 'market')) {
			// Авторизован аккаунт администратора, оператора или магазина

			// Инициализация буфера ошибок
			$this->errors['tasks'] ??= [];

			// Создание строк
			for ($i = 0, $parameters['cashiers'] = (int) $parameters['cashiers']; $i < $parameters['cashiers']; ++$i) model::create(work: 'Кассир', market: $this->account->type === 'market' ? $this->account->id : null, start: $parameters['start'], end: $parameters['end'], date: $parameters['date'], errors: $this->errors['tasks']);
			for ($i = 0, $parameters['displayers'] = (int) $parameters['displayers']; $i < $parameters['displayers']; ++$i) model::create(work: 'Выкладчик', market: $this->account->type === 'market' ? $this->account->id : null, start: $parameters['start'], end: $parameters['end'], date: $parameters['date'], errors: $this->errors['tasks']);
			for ($i = 0, $parameters['loaders'] = (int) $parameters['loaders']; $i < $parameters['loaders']; ++$i) model::create(work: 'Грузчик', market: $this->account->type === 'market' ? $this->account->id : null, start: $parameters['start'], end: $parameters['end'], date: $parameters['date'], errors: $this->errors['tasks']);
			for ($i = 0, $parameters['gastronomes'] = (int) $parameters['gastronomes']; $i < $parameters['gastronomes']; ++$i) model::create(work: 'Гастроном', market: $this->account->type === 'market' ? $this->account->id : null, start: $parameters['start'], end: $parameters['end'], date: $parameters['date'], errors: $this->errors['tasks']);

			// Запись заголовков ответа
			header('Content-Type: application/json');
			header('Content-Encoding: none');
			header('X-Accel-Buffering: no');

			// Инициализация буфера вывода
			ob_start();

			// Генерация ответа
			echo json_encode(
				[
					'errors' => self::parse_only_text($this->errors)
				]
			);

			// Запись заголовков ответа
			header('Content-Length: ' . ob_get_length());

			// Отправка и деинициализация буфера вывода
			ob_end_flush();
			flush();
		}
	}

	/**
	 * Прочитать
	 *
	 * @param array $parameters Параметры запроса
	 */
	public function read(array $parameters = []): ?string
	{
		if ($this->account->status()) {
			// Авторизован аккаунт

			// Реинициализация актуальной страницы
			if (isset($parameters['page'])) $this->session->write(['tasks' => ['page' => $parameters['page']]]);
			else if (empty($this->session->buffer[$_SERVER['INTERFACE']]['tasks']['page'])) $this->session->write(['tasks' => ['page' => 1]]);

			// Инициализация буфера AQL-выражения для инъекции фильтра по интервалу
			$interval = '';

			foreach (['from', 'to'] as $name) {
				// Перебор фильтров временного промежутка (И)

				// Инициализация значения (приоритет у cookie)
				if (empty($value = (int) ($_COOKIE["tasks_filter_$name"] ?? $this->session->buffer[$_SERVER['INTERFACE']]['tasks']['filters'][$name] ?? (($name === 'from') ? time() : strtotime('+1 month'))))) continue;

				// Генерация AQL-выражения для инъекции в строку запроса
				if ($name === 'from') $interval .= " && task.date >= $value";
				else if ($name === 'to') $interval .= " && task.date <= $value";
			}

			// Очистка от бинарных операторов сравнения с только одним операндом (крайние)
			$interval = trim(trim(trim($interval), '&&'));

			// Инициализация буфера AQL-выражения для инъекции фильтра по интервалу
			$polysemantic = '';

			foreach (['rating'] as $name) {
				// Перебор фильтров с произвольными значениями (И)

				// Инициализация значения (приоритет у cookie) 
				$value = $_COOKIE["tasks_filter_$name"] ?? $this->session->buffer[$_SERVER['INTERFACE']]['tasks']['filters'][$name] ?? null;

				// Найдено значение?
				if ($value === null) continue;

				// Генерация AQL-выражения для инъекции в строку запроса
				if ($name === 'rating' && $value > 0) $polysemantic .= " && task.rating >= $value";
			}

			// Очистка от бинарных операторов сравнения с только одним операндом (крайние)
			$polysemantic = trim(trim(trim($polysemantic), '&&'));

			// Инициализация допустимых статусов
			// @TODO Добавить инвертирование для waiting и unpublished
			$statuses = ['confirmed', 'waiting', 'published', 'unpublished', 'problematic', 'hided', 'completed'];

			// Инициализация буфера AQL-выражения для инъекции фильтра по статусам (И)
			$statuses_and = '';

			foreach ($statuses as $name) {
				// Перебор фильтров статусов (И)

				// Инициализация значения (приоритет у cookie) (отсутствие значения или значение 0 вызывают continue)
				if (empty($value = $_COOKIE["tasks_filter_$name"] ?? $this->session->buffer[$_SERVER['INTERFACE']]['tasks']['filters'][$name] ?? 0)) continue;

				// Генерация AQL-выражения для инъекции в строку запроса
				if ($value === '1') $statuses_and .= " && task.$name == true";
			}

			// Очистка от бинарных операторов сравнения с только одним операндом (крайние)
			$statuses_and = trim(trim(trim($statuses_and), '&&'));

			// Инициализация буфера AQL-выражения для инъекции фильтра по статусам (ИЛИ)
			$statuses_or = '';

			foreach ($statuses as $name) {
				// Перебор фильтров статусов (ИЛИ)

				// Инициализация значения (приоритет у cookie) (отсутствие значения или значение 0 вызывают continue)
				if (empty($value = $_COOKIE["tasks_filter_$name"] ?? $this->session->buffer[$_SERVER['INTERFACE']]['tasks']['filters'][$name] ?? 0)) continue;

				// Генерация AQL-выражения для инъекции в строку запроса
				if ($value === '2') $statuses_or .= " || task.$name == true";
			}

			// Очистка от бинарных операторов сравнения с только одним операндом (крайние)
			$statuses_or = trim(trim(trim($statuses_or), '||'));

			// Инициализация буфера с объёдинёнными буферами c AQL-выражениям "И" и "ИЛИ"
			$statuses_merged = (empty($statuses_and) ? '' : "($statuses_and)") . (empty($statuses_or) ? '' : (empty($statuses_and) ? '' : ' || ') . "($statuses_or)");

			// Инициализация общего буфера с AQL-выражениями
			$filters = '';

			// Объединение фильров в единую строку с AQL-выражениями для инъекции
			if (!empty($interval)) $filters .= $interval;
			if (!empty($statuses_merged)) $filters .= empty($filters) ? $statuses_merged : " && ($statuses_merged)";
			if (!empty($polysemantic)) $filters .= empty($filters) ? $polysemantic : " && $polysemantic";

			// Инициализация данных для генерации HTML-документа с таблицей
			if ($_SERVER['INTERFACE'] === 'worker')
				$this->view->rows = model::list(before: 'FILTER task.worker == "' . account::worker($this->account->number)?->id . '"' . empty($filters) ? null : " && ($filters)", page: (int) $this->session->buffer['worker']['tasks']['page']);
			else if ($_SERVER['INTERFACE'] === 'operator')
				$this->view->rows = model::list(before: empty($filters) ? null : "FILTER ($filters)", page: (int) $this->session->buffer['operator']['tasks']['page']);
			else if ($_SERVER['INTERFACE'] === 'market')
				$this->view->rows = model::list(before: 'FILTER task.market == "' . $this->market->id . '"' . empty($filters) ? null : " && ($filters)", page: (int) $this->session->buffer['market']['tasks']['page']);
			else if ($_SERVER['INTERFACE'] === 'administrator')
				$this->view->rows = model::list(before: empty($filters) ? null : "FILTER ($filters)", page: (int) $this->session->buffer['administrator']['tasks']['page']);
			else $this->view->rows = [];

			// Запись в cookie (только таким методом можно записать "hostonly: true")		
			setcookie(
				'tasks_page',
				(string) $this->session->buffer[$_SERVER['INTERFACE']]['tasks']['page'],
				[
					'expires' => strtotime('+1 hour'),
					'path' => '/',
					'secure' => true,
					'httponly' => false,
					'samesite' => 'strict'
				]
			);

			// Предобработка строк перед генерацией документа
			$this->view->rows = static::preprocessing($this->view->rows);

			// Запись в глобальную переменную шаблонизатора обрабатываемой страницы
			$this->view->page = $parameters['page'];

			// Инициализация блока
			return $this->view->render(DIRECTORY_SEPARATOR . 'elements' . DIRECTORY_SEPARATOR . 'tasks.html');
		}

		// Возврат (провал)
		return null;
	}

	/** 
	 * Предобработка строк перед генерацией документа
	 *
	 * @param array $rows Строки
	 *
	 * @return array Обработанные строки
	 */
	public static function preprocessing(array $rows): array
	{
		// Инициализация буфера прочитанных из базы данных строк
		$buffer = $rows;

		foreach ($rows as $number => $row) {
			// Перебор прочитанных из базы данных строк

			// Инициализация ярлыка
			$link = &$buffer[$number];

			// Конвертация времён
			if (!empty($row->task['start'])) $link->task = ['start' => datetime::createFromFormat('H:i', (string) $row->task['start'])] + $link->task;
			if (!empty($row->task['end'])) $link->task = ['end' => datetime::createFromFormat('H:i', (string) $row->task['end'])] + $link->task;

			// Инициализация буфера сгенерированных данных работы для шаблонизатора
			$generated = [];

			// Генерация данных работы для шаблонизатора
			if (
				$link->task['start'] instanceof datetime
				&& $link->task['end'] instanceof datetime
			) {
				$generated['hours'] = (float) $link->task['end']->diff($link->task['start'])->format('%H.%i');
				if ($generated['hours'] >= 6.5 && $generated['hours'] < 9) $generated['hours'] -= 0.5;
				else if ($generated['hours'] >= 9 && $generated['hours'] < 12.5) $generated['hours'] -= 1;
				else if ($generated['hours'] >= 12.5) $generated['hours'] -= 1.5;
			}
			if ($link->task['start'] instanceof datetime) $generated['start'] = $link->task['start']->format('H:i');
			if ($link->task['end'] instanceof datetime) $generated['end'] = $link->task['end']->format('H:i');

			// Запись из буфера сгенерированных данных работы для шаблонизатора в буфер переменных окружения шаблонизатора
			$link->task = ['generated' => $generated] + $link->task;
		}

		// Трансфер строк из модифицированного буфера процесса в буфер вывода
		return $buffer;
	}

	/**
	 * Заменить сотрудника или магазин
	 *
	 * @return void В буфер вывода JSON-документ с запрашиваемыми параметрами
	 */
	public function update(array $parameters = []): void
	{
		try {
			if ($this->account->status() && ($this->account->type === 'administrator' || $this->account->type === 'operator' || $this->account->type === 'market')) {
				// Авторизован аккаунт администратора, оператора или магазина

				if (($task = model::read('d._key == "' . $parameters['task'] . '"', amount: 1)) instanceof _document) {
					if (!empty($parameters['worker'])) {
						// Передан сотрудник
						if ($parameters['worker'] === 'delete') {
							// Удалить сотрудника

							// Удаление сотрудника
							$task->worker = null;

							if (_core::update($task)) {
								// Записано изменение в базу данных

								// Инициализация строки в глобальную переменную шаблонизатора
								$this->view->rows = static::preprocessing(model::list(before: 'FILTER task._key == "' . $parameters['task'] . '"', amount: 1));

								// Запись в глобальную переменную шаблонизатора обрабатываемой страницы (отключение)
								$this->view->page = null;

								// Запись заголовков ответа
								header('Content-Type: application/json');
								header('Content-Encoding: none');
								header('X-Accel-Buffering: no');

								// Инициализация буфера вывода
								ob_start();

								// Генерация ответа
								echo json_encode(
									[
										'updated' => true,
										'row' => $this->view->render(DIRECTORY_SEPARATOR . 'elements' . DIRECTORY_SEPARATOR . 'tasks.html'),
										'errors' => self::parse_only_text($this->errors)
									]
								);

								// Запись заголовков ответа
								header('Content-Length: ' . ob_get_length());

								// Отправка и деинициализация буфера вывода
								ob_end_flush();
								flush();
							} else throw new exception('Не удалось записать изменения в базу данных');
						} else {
							// Записать нового сотрудника

							if (($worker = worker::read('d.id == "' . $parameters['worker'] . '" && d.status == "active"', amount: 1)) instanceof _document) {
								// Найден сотрудник (запрашиваемый для записи сотрудник существует в базе данных)

								if ($task->worker !== $parameters['worker']) {
									// Идентификатор запрашиваемого сотрудника не равен актуальному

									// Запись сотрудника
									$task->worker = $worker->id;

									if (_core::update($task)) {
										// Записано изменение в базу данных

										// Инициализация строки в глобальную переменную шаблонизатора
										$this->view->rows = static::preprocessing(model::list(before: 'FILTER task._key == "' . $parameters['task'] . '"', amount: 1));

										// Запись в глобальную переменную шаблонизатора обрабатываемой страницы (отключение)
										$this->view->page = null;

										// Запись заголовков ответа
										header('Content-Type: application/json');
										header('Content-Encoding: none');
										header('X-Accel-Buffering: no');

										// Инициализация буфера вывода
										ob_start();

										// Генерация ответа
										echo json_encode(
											[
												'updated' => true,
												'row' => $this->view->render(DIRECTORY_SEPARATOR . 'elements' . DIRECTORY_SEPARATOR . 'tasks.html'),
												'errors' => self::parse_only_text($this->errors)
											]
										);

										// Запись заголовков ответа
										header('Content-Length: ' . ob_get_length());

										// Отправка и деинициализация буфера вывода
										ob_end_flush();
										flush();
									} else throw new exception('Не удалось записать изменения в базу данных');
								} else throw new exception('Сотрудник уже назначен');
							} else throw new exception('Не найден сотрудник');
						}
					} else if (!empty($parameters['market'])) {
						// Передан магазин

						if ($parameters['market'] === 'delete') {
							// Удалить магазин

							// Удаление магазина
							$task->market = null;

							if (_core::update($task)) {
								// Записано изменение в базу данных

								// Инициализация строки в глобальную переменную шаблонизатора
								$this->view->rows = static::preprocessing(model::list(before: 'FILTER task._key == "' . $parameters['task'] . '"', amount: 1));

								// Запись в глобальную переменную шаблонизатора обрабатываемой страницы (отключение)
								$this->view->page = null;

								// Запись заголовков ответа
								header('Content-Type: application/json');
								header('Content-Encoding: none');
								header('X-Accel-Buffering: no');

								// Инициализация буфера вывода
								ob_start();

								// Генерация ответа
								echo json_encode(
									[
										'updated' => true,
										'row' => $this->view->render(DIRECTORY_SEPARATOR . 'elements' . DIRECTORY_SEPARATOR . 'tasks.html'),
										'errors' => self::parse_only_text($this->errors)
									]
								);

								// Запись заголовков ответа
								header('Content-Length: ' . ob_get_length());

								// Отправка и деинициализация буфера вывода
								ob_end_flush();
								flush();
							} else throw new exception('Не удалось записать изменения в базу данных');
						} else {
							// Записать новый магазин

							if (($market = market::read('d.id == "' . $parameters['market'] . '" && d.status == "active"', amount: 1)) instanceof _document) {
								// Найден магазин (запрашиваемый для записи магазин существует в базе данных)

								if ($task->market !== $parameters['market']) {
									// Идентификатор запрашиваемого сотрудника не равен актуальному

									// Запись магазина
									$task->market = $market->id;

									if (_core::update($task)) {
										// Записано изменение в базу данных

										// Инициализация строки в глобальную переменную шаблонизатора
										$this->view->rows = static::preprocessing(model::list(before: 'FILTER task._key == "' . $parameters['task'] . '"', amount: 1));

										// Запись в глобальную переменную шаблонизатора обрабатываемой страницы (отключение)
										$this->view->page = null;

										// Запись заголовков ответа
										header('Content-Type: application/json');
										header('Content-Encoding: none');
										header('X-Accel-Buffering: no');

										// Инициализация буфера вывода
										ob_start();

										// Генерация ответа
										echo json_encode(
											[
												'updated' => true,
												'row' => $this->view->render(DIRECTORY_SEPARATOR . 'elements' . DIRECTORY_SEPARATOR . 'tasks.html'),
												'errors' => self::parse_only_text($this->errors)
											]
										);

										// Запись заголовков ответа
										header('Content-Length: ' . ob_get_length());

										// Отправка и деинициализация буфера вывода
										ob_end_flush();
										flush();
									} else throw new exception('Не удалось записать изменения в базу данных');
								} else throw new exception('Магазин уже назначен');
							} else throw new exception('Не найден магазин');
						}
					} else throw new exception('Не получены данные для записи');
				} else throw new exception('Не найдено задание');
			} else throw new exception('Вы не авторизованы');
		} catch (exception $e) {
			// Запись в реестр ошибок
			$this->errors[] = [
				'text' => $e->getMessage(),
				'file' => $e->getFile(),
				'line' => $e->getLine(),
				'stack' => $e->getTrace()
			];

			// Запись заголовков ответа
			header('Content-Type: application/json');
			header('Content-Encoding: none');
			header('X-Accel-Buffering: no');

			// Инициализация буфера вывода
			ob_start();

			// Генерация ответа
			echo json_encode(
				[
					'updated' => false,
					'errors' => self::parse_only_text($this->errors)
				]
			);

			// Запись заголовков ответа
			header('Content-Length: ' . ob_get_length());

			// Отправка и деинициализация буфера вывода
			ob_end_flush();
			flush();
		}
	}


	/**
	 * Прочитать данные задачи
	 *
	 * @param array $parameters Параметры запроса
	 */
	public function task(array $parameters = []): ?string
	{
		if ($this->account->status() && ($this->account->type === 'administrator' || $this->account->type === 'operator')) {
			// Авторизован аккаунт оператора

			// Инициализация данных сотрудника
			$this->view->task = model::read('d._key == "' . $parameters['task'] . '"', return: '{_key: d._key, created: d.created, updated: d.updated, confirmed: d.confirmed, hided: d.hided }')->getAll();

			if (!empty($this->view->task)) {
				// Найдены данные сотрудника

				// Инициализация буфера данных сотрудника
				$buffer = [];

				// Перевод ключей на русский язык
				foreach ($this->view->task as $key => $value)
					if (match ($key) {
						'created', 'updated', 'confirmed', 'hided', '_key' => true,
						default => false
					}) $buffer[$key] = [
						'label' => match ($key) {
							'created' => 'Создано',
							'updated' => 'Обновлено',
							'confirmed' => 'Подтверждено',
							'hided' => 'Скрыто',
							default => $key
						},
						'value' => $value
					];

				// Запись из буфера данных сотрудника
				$this->view->task = $buffer;

				// Возврат (успех)
				return $this->view->render(DIRECTORY_SEPARATOR . 'elements' . DIRECTORY_SEPARATOR . 'lists' . DIRECTORY_SEPARATOR . 'task.html');
			}
		}

		// Возврат (провал)
		return null;
	}


	/**
	 * Прочитать данные сотрудника
	 *
	 * @param array $parameters Параметры запроса
	 */
	public function worker(array $parameters = []): ?string
	{
		if ($this->account->status() && ($this->account->type === 'administrator' || $this->account->type === 'operator')) {
			// Авторизован аккаунт оператора

			// Инициализация данных сотрудника
			$this->view->worker = worker::read('d.id == "' . $parameters['worker'] . '"')->getAll();

			if (!empty($this->view->worker)) {
				// Найдены данные сотрудника

				// Инициализация буфера данных сотрудника
				$buffer = [];

				// Перевод ключей на русский язык
				foreach ($this->view->worker as $key => $value) $buffer[$key] = [
					'label' => match ($key) {
						'created' => 'Создано',
						'updated' => 'Обновлено',
						'id' => 'Идентификатор',
						'phone', 'number' => 'Номер',
						'birth' => 'Дата рождения',
						'address' => 'Адрес',
						'activity' => 'Деятельность',
						'passport' => 'Пасорт',
						'issued' => 'Место выдачи',
						'hiring' => 'Дата найма',
						'district' => 'Адрес регистрации',
						'department' => 'Отделение',
						'requisites' => 'Реквизиты',
						'fired' => 'Уволен',
						'tax' => 'ИНН',
						'city' => 'Город',
						'commentary' => 'Комментарий',
						'payment' => 'Получатель',
						default => $key
					},
					'value' => $value
				];

				// Запись из буфера данных
				$this->view->worker = $buffer;

				// Возврат (успех)
				return $this->view->render(DIRECTORY_SEPARATOR . 'elements' . DIRECTORY_SEPARATOR . 'lists' . DIRECTORY_SEPARATOR . 'worker.html');
			}
		}

		// Возврат (провал)
		return null;
	}

	/**
	 * Прочитать данные магазина
	 *
	 * @param array $parameters Параметры запроса
	 */
	public function market(array $parameters = []): ?string
	{
		if ($this->account->status() && ($this->account->type === 'administrator' || $this->account->type === 'operator')) {
			// Авторизован аккаунт оператора

			// Инициализация данных
			$this->view->market = market::read('CONCAT("K", SUBSTRING(d.id, 1)) == "' . $parameters['market'] . '"')?->getAll();

			if (!empty($this->view->market)) {
				// Найдены данные магазина

				// Инициализация буфера данных
				$buffer = [];

				// Перевод ключей на русский язык
				foreach ($this->view->market as $key => $value) $buffer[$key] = [
					'label' => match ($key) {
						'created' => 'Создано',
						'updated' => 'Обновлено',
						'id' => 'Идентификатор',
						'type' => 'Тип',
						'director' => 'Директор',
						'address' => 'Адрес',
						'city' => 'Город',
						default => $key
					},
					'value' => $value
				];

				// Запись из буфера данных
				$this->view->market = $buffer;

				// Возврат (успех)
				return $this->view->render(DIRECTORY_SEPARATOR . 'elements' . DIRECTORY_SEPARATOR . 'lists' . DIRECTORY_SEPARATOR . 'market.html');
			}
		}

		// Возврат (провал)
		return null;
	}

	/**
	 * Подтвердить (отклонить)
	 *
	 * @param array $parameters Параметры запроса
	 * 
	 * @return void В буфер вывода JSON-документ с запрашиваемыми параметрами
	 */
	public function confirm(array $parameters = []): void
	{
		if ($this->account->status() && ($this->account->type === 'administrator' || $this->account->type === 'operator')) {
			// Авторизован аккаунт оператора

			// Инициализация данных
			$task = model::read('d._key == "' . $parameters['task'] . '"');

			if ($task instanceof _document) {
				// Найдена заявка

				// Изменение статуса подтверждения
				$task->confirmed = !$task->confirmed;

				if (_core::update($task)) {
					// Записано изменение в базу данных

					// Инициализация строки в глобальную переменную шаблонизатора
					$this->view->rows = static::preprocessing(model::list(before: 'FILTER task._key == "' . $parameters['task'] . '"', amount: 1));

					// Запись в глобальную переменную шаблонизатора обрабатываемой страницы (отключение)
					$this->view->page = null;

					// Запись заголовков ответа
					header('Content-Type: application/json');
					header('Content-Encoding: none');
					header('X-Accel-Buffering: no');

					// Инициализация буфера вывода
					ob_start();

					// Генерация ответа
					echo json_encode(
						[
							'confirmed' => $this->view->rows[0]->task['confirmed'],
							'row' => $this->view->render(DIRECTORY_SEPARATOR . 'elements' . DIRECTORY_SEPARATOR . 'tasks.html'),
							'errors' => self::parse_only_text($this->errors)
						]
					);

					// Запись заголовков ответа
					header('Content-Length: ' . ob_get_length());

					// Отправка и деинициализация буфера вывода
					ob_end_flush();
					flush();
				}
			}
		}
	}

	/**
	 * Скрыть (показать)
	 *
	 * @param array $parameters Параметры запроса
	 * 
	 * @return void В буфер вывода JSON-документ с запрашиваемыми параметрами
	 */
	public function hide(array $parameters = []): void
	{
		if ($this->account->status() && ($this->account->type === 'administrator' || $this->account->type === 'operator')) {
			// Авторизован аккаунт оператора

			// Инициализация данных
			$task = model::read('d._key == "' . $parameters['task'] . '"');

			if ($task instanceof _document) {
				// Найдена заявка

				// Изменение статуса скрытия
				$task->hided = !$task->hided;

				if (_core::update($task)) {
					// Записано изменение в базу данных

					// Инициализация строки в глобальную переменную шаблонизатора
					$this->view->rows = static::preprocessing(model::list(before: 'FILTER task._key == "' . $parameters['task'] . '"', amount: 1));

					// Запись в глобальную переменную шаблонизатора обрабатываемой страницы (отключение)
					$this->view->page = null;

					// Запись заголовков ответа
					header('Content-Type: application/json');
					header('Content-Encoding: none');
					header('X-Accel-Buffering: no');

					// Инициализация буфера вывода
					ob_start();

					// Генерация ответа
					echo json_encode(
						[
							'hided' => $this->view->rows[0]->task['hided'],
							'row' => $this->view->render(DIRECTORY_SEPARATOR . 'elements' . DIRECTORY_SEPARATOR . 'tasks.html'),
							'errors' => self::parse_only_text($this->errors)
						]
					);

					// Запись заголовков ответа
					header('Content-Length: ' . ob_get_length());

					// Отправка и деинициализация буфера вывода
					ob_end_flush();
					flush();
				}
			}
		}
	}

	/**
	 * Удалить
	 *
	 * @param array $parameters Параметры запроса
	 * 
	 * @return void В буфер вывода JSON-документ с запрашиваемыми параметрами
	 */
	public function remove(array $parameters = []): void
	{
		if ($this->account->status() && ($this->account->type === 'administrator' || $this->account->type === 'operator')) {
			// Авторизован аккаунт оператора

			// Инициализация данных
			$task = model::read('d._key == "' . $parameters['task'] . '"');

			if ($task instanceof _document) {
				// Найдена заявка

				// Изменение статуса
				$task->status = 'deleted';

				if (_core::update($task)) {
					// Помечено как удалённое

					// Запись заголовков ответа
					header('Content-Type: application/json');
					header('Content-Encoding: none');
					header('X-Accel-Buffering: no');

					// Инициализация буфера вывода
					ob_start();

					// Генерация ответа
					echo json_encode(
						[
							'deleted' => true,
							'errors' => self::parse_only_text($this->errors)
						]
					);

					// Запись заголовков ответа
					header('Content-Length: ' . ob_get_length());

					// Отправка и деинициализация буфера вывода
					ob_end_flush();
					flush();
				}
			}
		}
	}

	/**
	 * Прочитать значение по названию
	 *
	 * @param array $parameters Параметры запроса
	 *
	 * @return mixed Значение из инстанции задачи в базе данных по имени
	 */
	public function value(array $parameters = []): mixed
	{
		if ($this->account->status() && ($this->account->type === 'administrator' || $this->account->type === 'operator' || $this->account->type === 'market')) {
			// Авторизован аккаунт оператора или магазина

			// Инициализация данных
			$task = model::read('d._key == "' . $parameters['task'] . '"');

			if ($task instanceof _document) {
				// Найдена заявка

				// Возврат (успех)
				return $task->{$parameters['name']} ?? null;
			}
		}

		// Возврат (провал)
		return null;
	}


	/**
	 * Прочитать данные работ для <datalist>
	 *
	 * @param array $parameters Параметры запроса
	 */
	public function works(array $parameters = []): ?string
	{
		if ($this->account->status() && ($this->account->type === 'administrator' || $this->account->type === 'operator' || $this->account->type === 'market')) {
			// Авторизован аккаунт оператора или магазина

			// Инициализация данных
			$this->view->task = model::read('d._key == "' . $parameters['task'] . '"');

			if ($this->view->task instanceof _document) {
				// Найдена заявка

				// Инициализация списка работ
				$this->view->works = ['Кассир', 'Выкладчик', 'Грузчик', 'Гастроном'];

				// Проверка на существование записанной в задаче работы в списке существующих работ и запись об этом в глобальную переменную шаблонизатора
				foreach ($this->view->works as $work) if ($this->view->task->work === $work) $this->view->exist = true;

				// Возврат (успех)
				return $this->view->render(DIRECTORY_SEPARATOR . 'lists' . DIRECTORY_SEPARATOR . 'works.html');
			}
		}

		// Возврат (провал)
		return null;
	}

	/**
	 * Записать тип работы
	 *
	 * @param array $parameters Параметры запроса
	 * 
	 * @return void В буфер вывода JSON-документ с запрашиваемыми параметрами
	 */
	public function work(array $parameters = []): void
	{
		if ($this->account->status() && ($this->account->type === 'administrator' || $this->account->type === 'operator' || $this->account->type === 'market')) {
			// Авторизован аккаунт оператора

			// Инициализация данных
			$task = model::read('d._key == "' . $parameters['task'] . '"');

			if ($task instanceof _document) {
				// Найдена заявка

				// Изменение статуса
				$task->work = match ($parameters['work']) {
					'Кассир', 'Выкладчик', 'Грузчик', 'Гастроном' => $parameters['work'],
					default => 'Кассир'
				};

				if (_core::update($task)) {
					// Записано в ArangoDB

					// Инициализация строки в глобальную переменную шаблонизатора
					$this->view->rows = static::preprocessing(model::list(before: 'FILTER task._key == "' . $parameters['task'] . '"', amount: 1));

					// Запись в глобальную переменную шаблонизатора обрабатываемой страницы (отключение)
					$this->view->page = null;

					// Запись заголовков ответа
					header('Content-Type: application/json');
					header('Content-Encoding: none');
					header('X-Accel-Buffering: no');

					// Инициализация буфера вывода
					ob_start();

					// Генерация ответа
					echo json_encode(
						[
							'writed' => true,
							'row' => $this->view->render(DIRECTORY_SEPARATOR . 'elements' . DIRECTORY_SEPARATOR . 'tasks.html'),
							'errors' => self::parse_only_text($this->errors)
						]
					);

					// Запись заголовков ответа
					header('Content-Length: ' . ob_get_length());

					// Отправка и деинициализация буфера вывода
					ob_end_flush();
					flush();
				}
			}
		}
	}

	/**
	 * Записать описание
	 *
	 * @param array $parameters Параметры запроса
	 * 
	 * @return void В буфер вывода JSON-документ с запрашиваемыми параметрами
	 */
	public function description(array $parameters = []): void
	{
		if ($this->account->status() && ($this->account->type === 'administrator' || $this->account->type === 'operator' || $this->account->type === 'market')) {
			// Авторизован аккаунт оператора

			// Инициализация данных
			$task = model::read('d._key == "' . $parameters['task'] . '"');

			if ($task instanceof _document) {
				// Найдена заявка

				// Изменение статуса
				$task->description = $parameters['description'];

				if (_core::update($task)) {
					// Записано в ArangoDB

					// Запись заголовков ответа
					header('Content-Type: application/json');
					header('Content-Encoding: none');
					header('X-Accel-Buffering: no');

					// Инициализация буфера вывода
					ob_start();

					// Генерация ответа
					echo json_encode(
						[
							'writed' => true,
							'errors' => self::parse_only_text($this->errors)
						]
					);

					// Запись заголовков ответа
					header('Content-Length: ' . ob_get_length());

					// Отправка и деинициализация буфера вывода
					ob_end_flush();
					flush();
				}
			}
		}
	}

	/**
	 * Записать дату и время
	 *
	 * @param array $parameters Параметры запроса
	 * 
	 * @return void В буфер вывода JSON-документ с запрашиваемыми параметрами
	 */
	public function date(array $parameters = []): void
	{
		if ($this->account->status() && ($this->account->type === 'administrator' || $this->account->type === 'operator' || $this->account->type === 'market')) {
			// Авторизован аккаунт оператора

			// Инициализация данных
			$task = model::read('d._key == "' . $parameters['task'] . '"');

			if ($task instanceof _document) {
				// Найдена заявка

				// Запись даты и времени
				if (!empty($parameters['date'])) $task->date = +$parameters['date'];
				if (!empty($parameters['start'])) $task->start = $parameters['start'];
				if (!empty($parameters['end'])) $task->end = $parameters['end'];

				if (_core::update($task)) {
					// Записано в ArangoDB

					// Инициализация строки в глобальную переменную шаблонизатора
					$this->view->rows = static::preprocessing(model::list(before: 'FILTER task._key == "' . $parameters['task'] . '"', amount: 1));

					// Запись в глобальную переменную шаблонизатора обрабатываемой страницы (отключение)
					$this->view->page = null;


					// Запись заголовков ответа
					header('Content-Type: application/json');
					header('Content-Encoding: none');
					header('X-Accel-Buffering: no');

					// Инициализация буфера вывода
					ob_start();

					// Генерация ответа
					echo json_encode(
						[
							'writed' => true,
							'row' => $this->view->render(DIRECTORY_SEPARATOR . 'elements' . DIRECTORY_SEPARATOR . 'tasks.html'),
							'errors' => self::parse_only_text($this->errors)
						]
					);

					// Запись заголовков ответа
					header('Content-Length: ' . ob_get_length());

					// Отправка и деинициализация буфера вывода
					ob_end_flush();
					flush();
				}
			}
		}
	}

	/**
	 * Записать комментарий
	 *
	 * @param array $parameters Параметры запроса
	 * 
	 * @return void В буфер вывода JSON-документ с запрашиваемыми параметрами
	 */
	public function commentary(array $parameters = []): void
	{
		if ($this->account->status() && ($this->account->type === 'administrator' || $this->account->type === 'operator' || $this->account->type === 'market')) {
			// Авторизован аккаунт оператора

			// Инициализация данных
			$task = model::read('d._key == "' . $parameters['task'] . '"');

			if ($task instanceof _document) {
				// Найдена заявка

				// Запись комментария
				$task->commentary = $parameters['commentary'];

				if (_core::update($task)) {
					// Записано в ArangoDB

					// Инициализация строки в глобальную переменную шаблонизатора
					$this->view->rows = static::preprocessing(model::list(before: 'FILTER task._key == "' . $parameters['task'] . '"', amount: 1));

					// Запись в глобальную переменную шаблонизатора обрабатываемой страницы (отключение)
					$this->view->page = null;

					// Запись заголовков ответа
					header('Content-Type: application/json');
					header('Content-Encoding: none');
					header('X-Accel-Buffering: no');

					// Инициализация буфера вывода
					ob_start();

					// Генерация ответа
					echo json_encode(
						[
							'writed' => true,
							'row' => $this->view->render(DIRECTORY_SEPARATOR . 'elements' . DIRECTORY_SEPARATOR . 'tasks.html'),
							'errors' => self::parse_only_text($this->errors)
						]
					);

					// Запись заголовков ответа
					header('Content-Length: ' . ob_get_length());

					// Отправка и деинициализация буфера вывода
					ob_end_flush();
					flush();
				}
			}
		}
	}
}
